﻿Imports Microsoft.Win32

Module RegistryManipulation
    Public HKEY_CLASSES_ROOT As String = "\\HKEY_CLASSES_ROOT"
    Public HKEY_CURRENT_USER As String = "\\HKEY_CURRENT_USER"
    Public HKEY_LOCAL_MACHINE As String = "\\HKEY_LOCAL_MACHINE"
    Public HKEY_USERS As String = "\\HKEY_USERS"
    Public HKEY_CURRENT_CONFIG As String = "\\HKEY_CURRENT_CONFIG"

    Public Function GetKey(ByVal KeyName As String) As RegistryKey

        Dim rootKey As String = KeyName.Substring(0, KeyName.IndexOf("\", 2))
        If Not (rootKey = HKEY_CLASSES_ROOT Or rootKey = HKEY_CURRENT_USER Or _
            rootKey = HKEY_LOCAL_MACHINE Or rootKey = HKEY_USERS Or _
            rootKey = HKEY_CURRENT_CONFIG) Then
            Err.Raise(-1, "Registry Manipulation", rootKey & " is an invalid root key")
        End If
        Dim baseRegistry As RegistryKey
        Select Case rootKey
            Case HKEY_CLASSES_ROOT
                baseRegistry = Registry.ClassesRoot
            Case HKEY_CURRENT_USER
                baseRegistry = My.Computer.Registry.CurrentUser
            Case HKEY_LOCAL_MACHINE
                baseRegistry = Registry.LocalMachine
            Case HKEY_USERS
                baseRegistry = Registry.Users
            Case HKEY_CURRENT_CONFIG
                baseRegistry = Registry.CurrentConfig
            Case Else
                baseRegistry = Registry.CurrentUser
        End Select
        Dim BaseKey As String = KeyName.Substring(KeyName.IndexOf("\", 2) + 1)
        Dim retKey As RegistryKey
        retKey = baseRegistry.OpenSubKey(BaseKey, True)
        If retKey Is Nothing Then
            If BaseKey.IndexOf("\") = 0 Then 'There are no further subkeys
                retKey = baseRegistry.CreateSubKey(BaseKey)
            Else
                Dim subBase As String = BaseKey.Substring(0, BaseKey.LastIndexOf("\"))
                Dim keyOnly As String = BaseKey.Substring(BaseKey.LastIndexOf("\") + 1)
                retKey = OpenKey(baseRegistry, subBase).CreateSubKey(keyOnly)
            End If
        End If
        Return retKey
    End Function

    Public Function OpenKey(ByVal BaseRegistry As RegistryKey, ByVal KeyName As String) As RegistryKey
        Dim retKey As RegistryKey
        retKey = BaseRegistry.OpenSubKey(KeyName, True)
        If retKey Is Nothing Then
            If KeyName.IndexOf("\") = 0 Then 'There are no further subkeys
                retKey = BaseRegistry.CreateSubKey(KeyName)
            Else
                Dim subBase As String = KeyName.Substring(0, KeyName.LastIndexOf("\"))
                Dim keyOnly As String = KeyName.Substring(KeyName.LastIndexOf("\") + 1)
                retKey = OpenKey(BaseRegistry, subBase).CreateSubKey(keyOnly)
            End If
        End If
        If retKey Is Nothing Then
            Err.Raise(-1, "Registry Manipulation", "Could not open " & KeyName)
        End If
        Return retKey
    End Function

    Public Function DeleteKey(ByVal KeyName As String) As Boolean
        Dim retSuccess As Boolean = False
        Dim rootKey As String = KeyName.Substring(0, KeyName.IndexOf("\", 2))
        If Not (rootKey = HKEY_CLASSES_ROOT Or rootKey = HKEY_CURRENT_USER Or _
            rootKey = HKEY_LOCAL_MACHINE Or rootKey = HKEY_USERS Or _
            rootKey = HKEY_CURRENT_CONFIG) Then
            Err.Raise(-1, "Registry Manipulation", rootKey & " is an invalid root key")
        End If
        Dim baseRegistry As RegistryKey
        Select Case rootKey
            Case HKEY_CLASSES_ROOT
                baseRegistry = Registry.ClassesRoot
            Case HKEY_CURRENT_USER
                baseRegistry = Registry.CurrentUser
            Case HKEY_LOCAL_MACHINE
                baseRegistry = Registry.LocalMachine
            Case HKEY_USERS
                baseRegistry = Registry.Users
            Case HKEY_CURRENT_CONFIG
                baseRegistry = Registry.CurrentConfig
            Case Else
                baseRegistry = Registry.CurrentUser
        End Select
        Dim BaseKey As String = KeyName.Substring(KeyName.IndexOf("\", 2) + 1)
        If BaseKey.IndexOf("\") = 0 Then ' we are deleting directly off the base registry
            baseRegistry.DeleteSubKey(BaseKey)
        Else
            Dim subKey As String = BaseKey.Substring(BaseKey.LastIndexOf("\") + 1)
            BaseKey = BaseKey.Substring(0, BaseKey.Length - subKey.Length - 1)
            Dim regKey As RegistryKey = OpenKey(baseRegistry, BaseKey)
            If Not regKey Is Nothing Then
                regKey.DeleteSubKey(subKey)
                retSuccess = True
            Else
                retSuccess = True
            End If
        End If
        Return retSuccess
    End Function

    Public Function SetRegistryValue(ByVal KeyName As String, ByVal EntryName As String, ByVal EntryValue As Object) As Boolean
        Dim regKey As RegistryKey
        regKey = GetKey(KeyName)
        regKey.SetValue(EntryName, EntryValue)
        Return True
    End Function
    Public Function GetRegistryValue(ByVal KeyName As String, ByVal EntryName As String, ByVal DefaultValue As Object) As Object
        Dim regKey As RegistryKey
        Dim retVal As Object
        regKey = GetKey(KeyName)
        retVal = regKey.GetValue(EntryName)
        If retVal Is Nothing Then
            retVal = DefaultValue
        End If
        Return retVal
    End Function
End Module
